;;;; fp.scm


(use srfi-1 fp2scheme readline stdlib.fp)

(define *trace* #f)

(define (usage code)
  (print #<<EOF
usage: fp [OPTION | FILENAME]

  -h             show this message
  -i             enter interactive mode
  -e EXPR        evaluate expression
  -c FILENAME    compile file to Scheme
  -t             enable trace mode
  -m             call `main' function on startup
  --             ignore all remaining arguments

EOF
;|#
)
  (exit code) )

(define (fp-repl)
  (when (or (##sys#fudge 12) (##sys#tty-port? (current-input-port)))
    (current-input-port (make-gnu-readline-port "\t"))
    (gnu-history-install-file-manager
     (string-append (or (getenv "HOME") ".") "/.fp.history")))
  (let loop ()
    (let ((ln (read-line)))
      (unless (eof-object? ln)
	(handle-exceptions ex
	    (print-error-message ex)
	  (let ((r (eval `(begin ,@(map (cut fp->scheme <> *trace*) (fp-parse ln))))))
	    (unless (eq? (void) r)
	      (fp:show r))))
	(loop) ) ) )
  (newline))

(handle-exceptions ex
    (begin
      (print-error-message ex)
      (exit 70) )
  (let ((args (command-line-arguments))
	(margs '())
	(main #f))
    (if (null? args)
	(fp-repl)
	(let loop ((args args))
	  (match args
	    (()
	     (if main
		 (fp:main (map string->symbol margs))
		 0))
	    (("-i") (fp-repl))
	    (("-t" . more)
	     (set! *trace* #t)
	     (loop more))
	    (("-e" str . more)
	     (eval `(begin ,@(map (cut fp->scheme <> *trace*) (fp-parse str))))
	     (loop more) )
	    (("-h" . _)
	     (usage 0) )
	    (("-c" f . more)
	     (for-each 
	      (lambda (x) 
		(pp (fp->scheme x *trace*))
		(newline) )
	      (if (not (string=? "-" f))
		  (call-with-input-file f fp-parse)
		  (fp-parse (current-input-port))))
	     (when main
	       (pp '(exit 
		     (let ((r (fp:main (map string->symbol (command-line-arguments)))))
		       (if (fixnum? r)
			   r
			   0) ) ) ) )
	     (exit 0) )
	    (((or "-c" "-e"))
	     (usage 1) )
	    (("--" . more)
	     (set! margs more)
	     (loop '()))
	    (("-m" . more)
	     (set! main #t)
	     (loop more) )
	    ((f . more)
	     (eval
	      `(begin 
		 ,@(map 
		    (cut fp->scheme <> *trace*)
		    (call-with-input-file f fp-parse))) )
	     (loop more) )
	    (_ (usage 1)))))))
